@@ -1,7 +1,6 @@
-;;; esh-module.el --- Eshell modules
+;;; esh-module.el --- Eshell modules  -*- lexical-binding:t -*-
 
-;; Copyright (C) 1999, 2000, 2002, 2003, 2004, 2005, 2006, 2007,
-;;   2008, 2009  Free Software Foundation, Inc.
+;; Copyright (C) 1999-2000, 2002-2019 Free Software Foundation, Inc.
 
 ;; Author: John Wiegley <johnw@gnu.org>
 ;; Keywords: processes
@@ -19,7 +18,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Code:
 
@@ -37,14 +36,16 @@ customizing the variable `eshell-modules-list'."
 
 ;; load the defgroup's for the standard extension modules, so that
 ;; documentation can be provided when the user customize's
-;; `eshell-modules-list'.
-(require 'esh-groups)
+;; `eshell-modules-list'.  We use "(progn (defgroup ..." in each file
+;; to force the autoloader into including the entire defgroup, rather
+;; than an abbreviated version.
+(load "esh-groups" nil 'nomessage)
 
 ;;; User Variables:
 
 (defcustom eshell-module-unload-hook
   '(eshell-unload-extension-modules)
-  "*A hook run when `eshell-module' is unloaded."
+  "A hook run when `eshell-module' is unloaded."
   :type 'hook
   :group 'eshell-module)
 
@@ -62,7 +63,7 @@ customizing the variable `eshell-modules-list'."
     eshell-script
     eshell-term
     eshell-unix)
-  "*A list of optional add-on modules to be loaded by Eshell.
+  "A list of optional add-on modules to be loaded by Eshell.
 Changes will only take effect in future Eshell buffers."
   :type (append
 	 (list 'set ':tag "Supported modules")
@@ -93,12 +94,11 @@ customization group.  Example: `eshell-cmpl' for that module."
 
 (defun eshell-unload-extension-modules ()
   "Unload any memory resident extension modules."
-  (eshell-for module (eshell-subgroups 'eshell-module)
+  (dolist (module (eshell-subgroups 'eshell-module))
     (if (featurep module)
 	(ignore-errors
 	  (message "Unloading %s..." (symbol-name module))
 	  (unload-feature module)
 	  (message "Unloading %s...done" (symbol-name module))))))
 
-;; arch-tag: 97a3fa16-9d08-40e6-bc2c-36bd70986507
 ;;; esh-module.el ends here
