@@ -1,10 +1,9 @@
-;;; soundex.el --- implement Soundex algorithm
+;;; soundex.el --- implement Soundex algorithm -*- lexical-binding: t -*-
 
-;; Copyright (C) 1993, 2001, 2002, 2003, 2004, 2005,
-;;   2006, 2007, 2008, 2009, 2010 Free Software Foundation, Inc.
+;; Copyright (C) 1993, 2001-2019 Free Software Foundation, Inc.
 
 ;; Author: Christian Plaunt <chris@bliss.berkeley.edu>
-;; Maintainer: FSF
+;; Maintainer: emacs-devel@gnu.org
 ;; Keywords: matching
 ;; Created: Sat May 15 14:48:18 1993
 
@@ -21,7 +20,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -30,7 +29,7 @@
 
 ;;; Code:
 
-(defvar soundex-alist
+(defconst soundex-alist
   '((?B . "1") (?F . "1") (?P . "1") (?V . "1")
     (?C . "2") (?G . "2") (?J . "2") (?K . "2") (?Q . "2") (?S . "2")
     (?X . "2") (?Z . "2") (?D . "3") (?T . "3") (?L . "4") (?M . "5")
@@ -61,16 +60,6 @@ and Searching\", Addison-Wesley (1973), pp. 391-392."
 	(substring (concat key "000") 0 4)
       key)))
 
-;(defvar soundex-test
-;  '("Euler" "Gauss" "Hilbert" "Knuth" "Lloyd" "Lukasiewicz"
-;    "Ellery" "Ghosh" "Heilbronn" "Kant" "Ladd" "Lissajous")
-;  "\n  Knuth's names to demonstrate the Soundex algorithm.")
-;
-;(mapcar 'soundex soundex-test)
-;("E460" "G200" "H416" "K530" "L300" "L222"
-; "E460" "G200" "H416" "K530" "L300" "L222")
-
 (provide 'soundex)
 
-;; arch-tag: b2615a98-feb7-430e-a717-171086738953
 ;;; soundex.el ends here
